﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SuperScooter
{
    class MMenu
    {
        #region Public Info
        //Types of menus you can create
        public enum MenuType { Root, Child };
        #endregion

        #region Menu Item Class
        /// <summary>
        /// Class for storing a menu item and its pertinant information.
        /// </summary>
        class menuItem
        {
            #region privates >.>;
            private string text;
            private Color color;
            private Vector2 pos;
            private bool selected;
            private bool absolute;
            private bool visible;
            #endregion

            #region properties
            public string Text
            {
                get { return text; }
                set { text = value; }
            }
            public Color DrawColor
            {
                get { return color; }
                set { color = value; }
            }
            public Vector2 Position
            {
                get { return pos; }
                set { pos = value;}
            }
            public bool Selected
            {
                get { return selected; }
                set { selected = value; }
            }
            public bool Absolute
            {
                get { return absolute; }
                set { absolute = value; }
            }
            public bool Visible
            {
                get { return visible; }
                set { visible = value; }
            }
            #endregion

            /// <summary>
            /// Creates a menu item object.
            /// (Intended to be a part of a Menu class.)
            /// </summary>
            /// <param name="text">Display text for the menu item.</param>
            /// <param name="position">Position the where the menu item will be drawn.</param>
            public menuItem(string text)
            {
                this.text = text;
                color = Color.White;
                selected = false;
                absolute = false;
                visible = true;
            }

            /// <summary>
            /// Creates a menu item object.
            /// (Intended to be a part of a Menu class.)
            /// </summary>
            /// <param name="text">Display text for the menu item.</param>
            /// <param name="position">Position the where the menu item will be drawn.</param>
            /// <param name="color">Color the text will be drawn in.</param>
            public menuItem(string text, Vector2 position, Color color)
            {
                this.text = text;
                this.color = color;
                pos = position;
                selected = false;
                absolute = true;
                visible = true;
            }

            //assumes static is false
            public void Draw(SpriteBatch sb, SpriteFont font)
            {
                if (visible)
                    sb.DrawString(font, text, new Vector2((int)pos.X, (int)pos.Y), color);
            }

            //assumes static is true
            public void Draw(SpriteBatch sb, SpriteFont font, SpriteFont normal)
            {
                if (visible)
                    sb.DrawString(font, text, new Vector2((int)(pos.X + normal.MeasureString(text).X / 2 - font.MeasureString(text).X / 2), (int)(pos.Y + normal.MeasureString(text).Y / 2 - font.MeasureString(text).Y / 2)), color);
            }

            public void Draw(SpriteBatch sb, SpriteFont font, Vector2 position)
            {
                if (visible)
                    sb.DrawString(font, text, new Vector2((int)(position.X), (int)(position.Y)), color);
            }
        };

        //List containing all the menu items currently in the menu
        private List<menuItem> menuList;
        #endregion

        #region Private Data Members
        private Vector2 menuPosition = new Vector2(0,0);
        private int menuItemPadding = 40;//absolute menu item padding
        private float menuItemPaddingCoefficient;//padding that scales with fonts
        private int currentSelection = 0;
        //private bool scalePaddingWithSelected = true;

        private Texture2D bg;
        private Texture2D bullet;
        private int bulletPadding = 10;
        private SpriteFont normalFont;
        private SpriteFont selectedFont;
        private Color normalColor = Color.White;
        private Color selectedColor = Color.White;
        #endregion

        #region Flags
        private bool visible = true;
        private bool staticItems = true;
        private bool absolutePadding = true;
        #endregion

        #region Public Properties
        /// <summary>
        /// Sets or returns the visibility flag for the menu.
        /// </summary>
        public bool Visible
        {
            get { return visible; }
            set { visible = value; }
        }

        /// <summary>
        /// Sets or returns the value for the StaticItems flag.
        /// True signifies that menu items will remain stationary.
        /// False signifies that menu item positions will shift relative to the selected item.
        /// </summary>
        public bool StaticItems
        {
            get { return staticItems; }
            set { staticItems = value; }
        }

        /// <summary>
        /// Sets or returns the flag drawing menu item padding.
        /// True draws padding based on MenuItemPadding and False draws padding based on MenuItemPaddingCoefficient.
        /// </summary>
        public bool AbsolutePadding
        {
            get { return absolutePadding; }
            set { absolutePadding = value; }
        }

        /// <summary>
        /// Returns the number of menu items in the list.
        /// </summary>
        public int MenuItemCount
        {
            get { return (menuList == null) ? 0 : menuList.Count(); }
        }

        /// <summary>
        /// Sets or returns the value for the top left hand anchor for the menu.
        /// Vector2 contains the X and Y coordinates where the menu starts on the screen.
        /// </summary>
        public Vector2 MenuPosition
        {
            get { return menuPosition; }
            set { menuPosition = value; }
        }

        /// <summary>
        /// Sets or returns the amount of space between menu items.
        /// </summary>
        public int MenuItemPadding
        {
            get { return menuItemPadding; }
            set { menuItemPadding = value; }
        }

        /// <summary>
        /// Sets or returns the coefficient for the amount of space between menu items relative to the sprite font.
        /// </summary>
        public float MenuItemPaddingCoefficient
        {
            get { return menuItemPaddingCoefficient; }
            set { menuItemPaddingCoefficient = value; }
        }

        /// <summary>
        /// Sets or returns the value for the padding in pixels around the bullet.
        /// Default is 10 pixels.
        /// </summary>
        public int BulletPadding
        {
            get { return bulletPadding; }
            set { bulletPadding = value; }
        }

        /// <summary>
        /// Sets or returns the default font used for drawing the menu items.
        /// </summary>
        public SpriteFont NormalFont
        {
            get { return normalFont; }
            set { normalFont = value; }
        }

        /// <summary>
        /// Sets or returns the default font used for drawing the selected menu item.
        /// </summary>
        public SpriteFont SelectedFont
        {
            get { return selectedFont; }
            set { selectedFont = value; }
        }
        /// <summary>
        /// Sets or returns the color of non-selected menu items.
        /// </summary>
        public Color NormalColor
        {
            get { return normalColor; }
            set { normalColor = value; }
        }

        /// <summary>
        /// Sets or returns the color of selected menu items.
        /// </summary>
        public Color SelectedColor
        {
            get { return selectedColor; }
            set { selectedColor = value; }
        }
        /// <summary>
        /// Sets or returns the index of the currently selected menu item.
        /// Attempting to set currently selected menu item to a value less than zero results in the first menu item.
        /// Attempting to set currently selected menu item to a value greater than the last item in the list results in that last item.
        /// </summary>
        public int CurrentlySelectedIndex
        {
            get { return currentSelection; }
            set
            {
                menuList[currentSelection].Selected = false;
                currentSelection = (int)MathHelper.Clamp(value, 0, menuList.Count() - 1);
                menuList[currentSelection].Selected = true;
            }
        }

        /// <summary>
        /// Sets or returns the string of the currently selected menu item.
        /// Attempting to set currently selected menu item to a string that is not in the menu list will not change anything.
        /// </summary>
        public string CurrentlySelectedItem
        {
            get { return menuList[currentSelection].Text; }
            set
            {
                foreach (menuItem m in menuList)
                {
                    if (m.Text == value)
                        currentSelection = menuList.IndexOf(m);
                }
            }
        }

        public Texture2D BGImage
        {
            get { return bg; }
        }
        #endregion

        public MMenu(Vector2 position, Texture2D backgroundTexture, Texture2D bulletTexture, SpriteFont normalFont, SpriteFont selectedFont)
        {
            menuList = new List<menuItem>();
            currentSelection = 0;
            menuPosition = position;
            bg = backgroundTexture;
            bullet = bulletTexture;
            this.normalFont = normalFont;
            this.selectedFont = selectedFont;
        }

        public void AddMenuItem(string displayText)
        {
            menuItem newItem = new menuItem(displayText);
            newItem.Selected = (menuList.Count() == 0) ? true : false;
            menuList.Add(newItem);
        }

        public void AddMenuItem(string displayText, Vector2 displayPosition, Color displayColor)
        {
            menuItem newItem = new menuItem(displayText, displayPosition, displayColor);
            newItem.Selected = (menuList.Count() == 0) ? true : false;
            menuList.Add(newItem);
        }

        public void SetSelected(string text)
        {
            foreach (menuItem m in menuList)
            {
                if (m.Text == text)
                {
                    m.Selected = true;
                    menuList[currentSelection].Selected = false;
                    CurrentlySelectedItem = text;
                }
            }
        }

        public void SelectPrev()
        {
            menuList[currentSelection].Selected = false;
            do
            {
                currentSelection--;
                if (currentSelection < 0)
                    currentSelection = menuList.Count() - 1;
            }
            while(!menuList[currentSelection].Visible);
            menuList[currentSelection].Selected = true;
        }

        public void SelectNext()
        {
            menuList[currentSelection].Selected = false;
            do
            {
                currentSelection++;
                if (currentSelection >= menuList.Count())
                    currentSelection = 0;
            } while (!menuList[currentSelection].Visible);
            menuList[currentSelection].Selected = true;
        }

        public bool MenuItemVisibility(string text)
        {
            foreach (menuItem m in menuList)
            {
                if (m.Text == text)
                    return m.Visible;
            }

            //if no menu item is found to match text, return false
            return false;
        }

        public bool MenuItemVisibility(int index)
        {
            if (index > 0 && index < menuList.Count())
            {
                return menuList[index].Visible;
            }

            //if no menu item is found to match text, return false
            return false;
        }

        public void MenuItemVisibility(string text, bool visibility)
        {
            foreach (menuItem m in menuList)
            {
                if (m.Text == text)
                {
                    m.Visible = visibility;
                    if (!visibility)
                    {
                        m.Selected = false;
                        SelectNext();
                    }
                }
            }
        }

        public void MenuItemVisibility(int index, bool visibility)
        {
            if (index > 0 && index < menuList.Count())
            {
                menuList[index].Visible = visibility;
                if (!visibility)
                {
                    menuList[index].Selected = false;
                    SelectNext();
                }
            }
        }

        public void Draw(SpriteBatch spriteBatch)
        {
            if (visible)
            {
                if (bg != null)
                    spriteBatch.Draw(bg, new Vector2(0, 0), Color.White);

                Vector2 menuBottom = menuPosition;
                foreach (menuItem i in menuList)
                {
                    SpriteFont font = (i.Selected) ? selectedFont : normalFont;

                    if (i.Selected)
                    {
                        //draw acorn
                        i.DrawColor = selectedColor;
                    }
                    else
                    {
                        //don't draw the acorn
                        i.DrawColor = normalColor;
                    }

                    if (i.Absolute)//if the menuItem has an absolute position, don't pass it a draw position
                        i.Draw(spriteBatch, font, normalFont);
                    else//if the current menuItem doesn't have an absolute position, calculate one for it
                    {
                        //initialize menu item displacement to the menu's x position and y position plus half of the item's height
                        Vector2 menuItemPosition = menuBottom;

                        if (staticItems)
                        {
                            int diff = (!i.Selected) ? 0 : (int)(stringDims(i.Text, selectedFont).Y - stringDims(i.Text, normalFont).Y) / 2;
                            menuItemPosition.Y -= diff;
                            i.Draw(spriteBatch, font, menuItemPosition);
                            menuBottom.Y += stringDims(i.Text, normalFont).Y + getMenuItemPadding();
                        }
                        else
                        {
                            i.Draw(spriteBatch, font, menuItemPosition);
                            menuBottom.Y += stringDims(i.Text, font).Y + getMenuItemPadding();
                        }
                    }
                }
            }
        }

        private Vector2 stringDims(string s, SpriteFont sf)
        {
            return sf.MeasureString(s);
        }

        private float getMenuItemPadding()
        {
            if (absolutePadding)
                return menuItemPadding;
            else
                return stringDims(" ", normalFont).Y * menuItemPaddingCoefficient;
        }

    }
}
