﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace SuperScooter
{
    /// <summary>
    /// Sprite class used for animating an entity on the screen.
    /// </summary>
    class Sprite
    {
        //constants
        public const int FRAME_LENGTH = 100;//frame length in miliseconds
        public enum Direction {LEFT, RIGHT};

        /// <summary>
        /// Structure for storing dimension pairs.
        /// </summary>
        public struct dimensions
        {
            public int width;
            public int height;
        };

        //private variables
        private Game game;
        private Texture2D sourceImage;
        private dimensions frameDimensions;//in pixels
        private int horizontleGap;//in pixels
        private int stateCount;
        private int currentState;
        private int deathState;
        private int standingState;
        private Direction sourceDirection;
        private Direction currentDirection;
        private int elapsed = 0;
        private float alphaValue = 1.0f;
        //private bool flipped = false;
        private float rotation = 0;

        #region Properties
        /// <summary>
        /// Returns the width of the sprite's frame.
        /// </summary>
        public int SpriteWidth
        {
            get { return frameDimensions.width; }
        }
        /// <summary>
        /// Returns the height of the sprite's frame.
        /// </summary>
        public int SpriteHeight
        {
            get { return frameDimensions.height; }
        }
        /// <summary>
        /// Returns the number of states the sprite has.
        /// </summary>
        public int StateCount
        {
            get { return stateCount; }
        }
        /// <summary>
        /// Returns the texture the sprite is using for drawing.
        /// </summary>
        public Texture2D SourceImage
        {
            get { return sourceImage; }
        }
        /// <summary>
        /// Returns the current state of the sprite; this can be interpreted as an array index for frames in the sprite texture.
        /// </summary>
        public int CurrentState
        {
            get { return currentState; }
        }
        /// <summary>
        /// Get or set the designated death state for the sprite.
        /// </summary>
        public int DeathState
        {
            get { return deathState; }
            set
            {
                if (value >= 0 && value < stateCount)
                    deathState = value;
            }
        }
        /// <summary>
        /// Get or set the designated standing still state for the sprite.
        /// </summary>
        public int StandingState
        {
            get { return standingState; }
            set
            {
                if (value >= 0 && value < stateCount)
                    standingState = value;
            }
        }
        /// <summary>
        /// Returns the value passed in for the facing direction of the sprite texture.
        /// </summary>
        public Direction SourceDirection
        {
            get { return sourceDirection; }
        }
        /// <summary>
        /// Returns the current facing direction of the sprite.
        /// </summary>
        public Direction CurrentDirection
        {
            get { return currentDirection; }
            set { currentDirection = value; }
        }
        public float AlphaValue
        {
            get { return alphaValue; }
            set { alphaValue = MathHelper.Clamp(value, 0f, 1f); }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Sprite Class expects a source image with horizontally adjacent frames through which the sprite is to animate.
        /// </summary>
        /// <param name="game">Current game object.</param>
        /// <param name="imageSource">Texture2D that contains the source image for the sprite.</param>
        /// <param name="imageFacingDirection">Direction that the sprite is facing in the source image.</param>
        /// <param name="initialMovementDirection">Direction that the sprite is moving initially on the screen.</param>
        /// <param name="frameWidth">Determines the width of each sprite frame in pixels.</param>
        /// <param name="frameHeight">Determines the height of each sprite frame in pixels.</param>
        /// <param name="horizontleSpaceSize">Determines the size in pixels of the space between frames horizontally.</param>
        /// <param name="numberOfStates">Number of states contained in the source image.</param>
        /// <param name="deadState">Defines the state that represents the sprite's death image.</param>
        /// <param name="stillState">Defines the state that represents the sprite's standing still image.</param>
        public Sprite(Game game, Texture2D imageSource, Direction imageFacingDirection, Direction initialMovementDirection, int frameWidth, int frameHeight, int horizontleSpaceSize, int numberOfStates, int deadState, int stillState)
        {
            this.game = game;
            sourceImage = imageSource;
            sourceDirection = imageFacingDirection;
            currentDirection = initialMovementDirection;
            frameDimensions.width = frameWidth;
            frameDimensions.height = frameHeight;
            horizontleGap = horizontleSpaceSize;
            stateCount = numberOfStates;
            deathState = deadState;
            standingState = stillState;
        }
        #endregion

        #region Public Functions (ChangeState, StandSprite, KillSprite, ...)
        /// <summary>
        /// Changes the visible state of the sprite, animating it while avoiding the death and standing states.
        /// </summary>
        public void ChangeState()
        {
            if (currentState != deathState)//if the sprite is dead, don't change it
            {
                System.TimeSpan elapsedTime = game.TargetElapsedTime;
                elapsed += elapsedTime.Milliseconds;
                if (elapsed > FRAME_LENGTH)
                {
                    elapsed = 0;
                    do
                    {
                        currentState++;
                        if (currentState >= stateCount)//wrap the states around
                            currentState = 0;
                    }
                    while (currentState == deathState || currentState == standingState);//don't stop on the deathState or the standState                  
                }
            }
        }

        /// <summary>
        /// Reverses the direction the sprite is facing.
        /// </summary>
        public void ChangeDirection()
        {
            currentDirection = (currentDirection == Direction.RIGHT) ? Direction.LEFT : Direction.RIGHT;
        }

        /// <summary>
        /// Causes the sprite to be drawn flipped vertically.
        /// </summary>
        public void FlipSpriteUpSideDown()
        {
            rotation = MathHelper.Pi;
            ChangeDirection();
        }

        /// <summary>
        /// Causes the sprite to be drawn right side up.
        /// </summary>
        public void FlipSpriteRightSideUp()
        {
            rotation = 0f;
            ChangeDirection();
        }

        /// <summary>
        /// Puts the sprite in the standing still state.
        /// </summary>
        public void StandSprite()
        {
            if(standingState >= 0 && standingState < stateCount)
                currentState = standingState;
        }
        /// <summary>
        /// Puts the sprite in the death state.
        /// </summary>
        public void KillSprite()
        {
            if (deathState >= 0 && deathState < stateCount)
                currentState = deathState;
        }
        #endregion

        #region Draw Functions
        /// <summary>
        /// Draw function for the Sprite Class.
        /// </summary>
        /// <param name="spriteBatch">SpriteBatch to which to draw.</param>
        /// <param name="x">X position where the image will be drawn.</param>
        /// <param name="y">Y position where the image will be drawn.</param>
        public void Draw(SpriteBatch spriteBatch, int x, int y)
        {
            Rectangle rect = getCurrentDrawRect();
            if (currentDirection == sourceDirection) //draw it normally
                spriteBatch.Draw(sourceImage, new Rectangle(x + Tile.SIZE / 2, y + Tile.SIZE / 2, frameDimensions.width, frameDimensions.height), rect, new Color(255, 255, 255, alphaValue), rotation, new Vector2(Tile.SIZE / 2, Tile.SIZE / 2), SpriteEffects.None, 0);
            else                                     //draw it flipped horizontally
                spriteBatch.Draw(sourceImage, new Rectangle(x + Tile.SIZE / 2, y + Tile.SIZE / 2, frameDimensions.width, frameDimensions.height), rect, new Color(255, 255, 255, alphaValue), rotation, new Vector2(Tile.SIZE / 2, Tile.SIZE / 2), SpriteEffects.FlipHorizontally, 0);
        }

        /// <summary>
        /// Draw function for the Sprite Class.
        /// </summary>
        /// <param name="spriteBatch">SpriteBatch to which to draw.</param>
        /// <param name="x">X position where the image will be drawn.</param>
        /// <param name="y">Y position where the image will be drawn.</param>
        /// <param name="color">Draw color for the sprite.</param>
        public void Draw(SpriteBatch spriteBatch, int x, int y, Color color)
        {
            Rectangle rect = getCurrentDrawRect();
            if (currentDirection == sourceDirection) //draw it normally
                spriteBatch.Draw(sourceImage, new Rectangle(x + Tile.SIZE / 2, y + Tile.SIZE / 2, frameDimensions.width, frameDimensions.height), rect, color, rotation, new Vector2(Tile.SIZE / 2, Tile.SIZE / 2), SpriteEffects.None, 0);
            else                                     //draw it flipped horizontally
                spriteBatch.Draw(sourceImage, new Rectangle(x + Tile.SIZE / 2, y + Tile.SIZE / 2, frameDimensions.width, frameDimensions.height), rect, color, rotation, new Vector2(Tile.SIZE / 2, Tile.SIZE / 2), SpriteEffects.FlipHorizontally, 0);
        }
        #endregion

        #region Auxilary Functions
        /// <summary>
        /// Gets the current area of the sprite's source texture to be drawn to the screen.
        /// </summary>
        /// <returns>The area of the source texture that will be drawn to the screen based on the sprite's current state and dimensions.</returns>
        private Rectangle getCurrentDrawRect()
        {
            return new Rectangle(currentState * (frameDimensions.width + horizontleGap), 0, frameDimensions.width, frameDimensions.height);
        }
        #endregion

    }
}
